IMPLEMENTATION MODULE BIOScalls;

FROM SYSTEM IMPORT ADDRESS , CODE , SETREG , REGISTER;


PROCEDURE getmpb( pmpb : ADDRESS );  (* BIOS call #0 *)

BEGIN;

  SETREG(7,pmpb);                (* put table address into register D7 *)
  CODE( 2F07H );                 (* push longword D7 onto stack *)
  CODE(3F3CH,0,4E4DH);   (* push call number onto stack and call TRAP #13 *)

END getmpb;


PROCEDURE bconstat ( dev : CARDINAL ) : LONGINT;  (* BIOS call #1 *)

BEGIN;

  CODE( 7000H );                 (* clear D0 register *)
  SETREG(7,ADDRESS(dev));        (* put device number into register D7 *)
  CODE( 3F07H );                 (* push word D7 onto stack *)
  CODE(3F3CH,1,4E4DH);   (* push call number onto stack and call TRAP #13 *)
  RETURN LONGINT(REGISTER(0));   (* return status of character device *)

END bconstat;


PROCEDURE bconin( dev : CARDINAL ) : LONGINT;  (* BIOS call #2 *)

BEGIN;

  CODE ( 7000H );                (* clear D0 register *)
  SETREG(7,ADDRESS(dev));        (* put device number into register D7 *)
  CODE( 3F07H );                 (* push word D7 onto stack *)
  CODE(3F3CH,2,4E4DH);   (* push call number onto stack and call TRAP #13 *)
  RETURN LONGINT(REGISTER(0));   (* return input character longword *)

END bconin;


PROCEDURE bconout( dev : CARDINAL; ByteOut : CARDINAL );  (* BIOS call #3 *)

BEGIN;

  SETREG(7,ADDRESS(ByteOut));    (* put character into register D7 *)
  CODE( 3F07H );                 (* push word D7 onto stack *)
  SETREG(7,ADDRESS(dev));        (* put device number into register D7 *)
  CODE( 3F07H );                 (* push word D7 onto stack *)
  CODE(3F3CH,3,4E4DH);   (* push call number onto stack and call TRAP #13 *)

END bconout;


PROCEDURE rwabs( buff : ADDRESS ; rwflag, count, recno, dev : CARDINAL )
: LONGINT;  (* BIOS call #4 *)

BEGIN;

  CODE ( 7000H );              (* clear D0 register *)
  SETREG(7,ADDRESS(dev));      (* put mass storage device number into *)
                                   (* register D7 *)
  CODE( 3F07H );               (* push word D7 onto stack *)
  SETREG(7,ADDRESS(recno));    (* put starting logical sector number into *)
                                    (* register D7 *)
  CODE( 3F07H );               (* push word D7 onto stack *)
  SETREG(7,ADDRESS(count));    (* put sector count into register D7 *)
  CODE( 3F07H );               (* push word D7 onto stack *)
  SETREG(7,buff);              (* put buffer address into register D7 *)
  CODE( 2F07H );               (* push longword D7 onto stack *)
  SETREG(7,ADDRESS(rwflag));   (* put read/write flag into register D7 *)
  CODE( 3F07H );               (* push word D7 onto stack *)
  CODE(3F3CH,4,4E4DH);   (* push call number onto stack and call TRAP #13 *)
  RETURN LONGINT(REGISTER(0)); (* return mass storage device error *)
                                  (* status word *)
END rwabs;


PROCEDURE setexc( vecnum : CARDINAL; vec : ADDRESS): ADDRESS;
                                                          (* BIOS call #5 *)
BEGIN;

  CODE ( 7000H );                (* clear D0 register *)
  SETREG(7,vec);                 (* put vector address into register D7 *) 
  CODE ( 2F07H );                (* push longword D7 onto stack *)
  SETREG(7,ADDRESS(vecnum));     (* put vector number into register D7 *)
  CODE( 3F07H );                 (* push word D7 onto stack *)
  CODE(3F3CH,5,4E4DH);   (* push call number onto stack and call TRAP #13 *)
  RETURN REGISTER(0);            (* return old vector address *)

END setexc;


PROCEDURE tickcal(): LONGINT;  (* BIOS call #6 *)

BEGIN;

  CODE ( 7000H );                (* clear D0 register *)
  CODE(3F3CH,6,4E4DH);   (* push call number onto stack and call TRAP #13 *)
  RETURN LONGINT(REGISTER(0));   (* return system clock in milliseconds *)

END tickcal;


PROCEDURE getbpb( dev : CARDINAL ): ADDRESS;  (* BIOS call #7 *)

BEGIN;

  CODE ( 7000H );                (* clear D0 register *)
  SETREG(7,ADDRESS(dev));        (* put mass storage device number into *)
                                    (* register D7 *)
  CODE( 3F07H );                 (* push word D7 onto stack *)
  CODE(3F3CH,7,4E4DH);   (* push call number onto stack and call TRAP #13 *)
  RETURN REGISTER(0);            (* return BIOS parameter block pointer *)

END getbpb;


PROCEDURE bcostat( dev : CARDINAL ): LONGINT;  (* BIOS call #8 *)

BEGIN;

  CODE ( 7000H );                (* clear D0 register *)
  SETREG(7,ADDRESS(dev));        (* put device number into register D7 *)
  CODE( 3F07H );                 (* push word D7 onto stack *)
  CODE(3F3CH,8,4E4DH);   (* push call number onto stack and call TRAP #13 *)
  RETURN LONGINT(REGISTER(0));   (* return device output status word *)

END bcostat;


PROCEDURE mediach( dev :CARDINAL ): LONGINT;  (* BIOS call #9 *)

BEGIN;

  CODE ( 7000H );                (* clear D0 register *)
  SETREG(7,ADDRESS(dev));        (* put mass storage device number into *)
                                    (* register D7 *)
  CODE( 3F07H );                 (* push word D7 onto stack *)
  CODE(3F3CH,9,4E4DH);   (* push call number onto stack and call TRAP #13 *)
  RETURN LONGINT(REGISTER(0));   (* return mass storage device media *)
                                    (* status word *)
END mediach;
  

PROCEDURE drvmap(): LONGCARD;  (* BIOS call #10 *)

BEGIN;

  CODE ( 7000H );                (* clear D0 register *)
  CODE(3F3CH,10,4E4DH);  (* push call number onto stack and call TRAP #13 *)
  RETURN LONGCARD(REGISTER(0));  (* returns longword that maps active *)
                                    (* storage units available *)
END drvmap;


PROCEDURE kbshift( mode : LONGINT ) : LONGINT;  (* BIOS call *11 *)

BEGIN;

  CODE ( 7000H );                (* clear D0 register *)
  SETREG(7,ADDRESS(mode));       (* put mode into register D7 *)
  CODE( 2F07H );                 (* push longword D7 onto stack *)
  CODE(3F3CH,11,4E4DH);  (* push call number onto stack and call TRAP #13 *)
  RETURN LONGINT(REGISTER(0));   (* return shift key status word *)

END kbshift;


END BIOScalls.
QQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQ